# Autogenerated by nbdev

d = { 'settings': { 'allowed_cell_metadata_keys': '',
                'allowed_metadata_keys': '',
                'audience': 'Developers',
                'author': 'Scott Hawley',
                'author_email': 'scott.hawley@belmont.edu',
                'black_formatting': 'False',
                'branch': 'main',
                'clean_ids': 'True',
                'console_scripts': 'chunkadelic=aeiou.chunkadelic:main spectrofu=aeiou.spectrofu:main',
                'copyright': 'Put your copyright info here',
                'custom_quarto_yml': 'False',
                'custom_sidebar': 'False',
                'description': 'audio engineering i/o utils',
                'doc_baseurl': '/aeiou/',
                'doc_host': 'https://drscotthawley.github.io',
                'doc_path': '_docs',
                'git_url': 'https://github.com/drscotthawley/aeiou/',
                'host': 'github',
                'jupyter_hooks': 'True',
                'keywords': 'nbdev',
                'language': 'English',
                'lib_name': 'aeiou',
                'lib_path': 'aeiou',
                'license': 'apache2',
                'min_python': '3.7',
                'nbs_path': '.',
                'readme_nb': 'index.ipynb',
                'recursive': 'False',
                'repo': 'aeiou',
                'requirements': 'fastcore pandas numpy torch torchvision torchaudio wandb matplotlib pillow tqdm librosa>=0.8.1 einops accelerate',
                'status': '2',
                'title': 'aeiou',
                'tst_flags': 'notest',
                'user': 'drscotthawley',
                'version': '0.0.8'},
  'syms': { 'aeiou.chunkadelic': { 'aeiou.chunkadelic.blow_chunks': 'https://drscotthawley.github.io/aeiou/chunkadelic.html#blow_chunks',
                                   'aeiou.chunkadelic.main': 'https://drscotthawley.github.io/aeiou/chunkadelic.html#main',
                                   'aeiou.chunkadelic.process_one_file': 'https://drscotthawley.github.io/aeiou/chunkadelic.html#process_one_file'},
            'aeiou.core': { 'aeiou.core.fast_scandir': 'https://drscotthawley.github.io/aeiou/core.html#fast_scandir',
                            'aeiou.core.get_audio_filenames': 'https://drscotthawley.github.io/aeiou/core.html#get_audio_filenames',
                            'aeiou.core.is_silence': 'https://drscotthawley.github.io/aeiou/core.html#is_silence',
                            'aeiou.core.load_audio': 'https://drscotthawley.github.io/aeiou/core.html#load_audio',
                            'aeiou.core.makedir': 'https://drscotthawley.github.io/aeiou/core.html#makedir'},
            'aeiou.datasets': { 'aeiou.datasets.AudioDataset': 'https://drscotthawley.github.io/aeiou/datasets.html#audiodataset',
                                'aeiou.datasets.AudioDataset.get_data_range': 'https://drscotthawley.github.io/aeiou/datasets.html#audiodataset.get_data_range',
                                'aeiou.datasets.AudioDataset.get_next_chunk': 'https://drscotthawley.github.io/aeiou/datasets.html#audiodataset.get_next_chunk',
                                'aeiou.datasets.AudioDataset.load_file_ind': 'https://drscotthawley.github.io/aeiou/datasets.html#audiodataset.load_file_ind',
                                'aeiou.datasets.AudioDataset.preload_files': 'https://drscotthawley.github.io/aeiou/datasets.html#audiodataset.preload_files',
                                'aeiou.datasets.FillTheNoise': 'https://drscotthawley.github.io/aeiou/datasets.html#fillthenoise',
                                'aeiou.datasets.Mono': 'https://drscotthawley.github.io/aeiou/datasets.html#mono',
                                'aeiou.datasets.NormInputs': 'https://drscotthawley.github.io/aeiou/datasets.html#norminputs',
                                'aeiou.datasets.PadCrop': 'https://drscotthawley.github.io/aeiou/datasets.html#padcrop',
                                'aeiou.datasets.PadCrop.draw_chunk': 'https://drscotthawley.github.io/aeiou/datasets.html#padcrop.draw_chunk',
                                'aeiou.datasets.PhaseFlipper': 'https://drscotthawley.github.io/aeiou/datasets.html#phaseflipper',
                                'aeiou.datasets.RandPool': 'https://drscotthawley.github.io/aeiou/datasets.html#randpool',
                                'aeiou.datasets.RandomGain': 'https://drscotthawley.github.io/aeiou/datasets.html#randomgain',
                                'aeiou.datasets.Stereo': 'https://drscotthawley.github.io/aeiou/datasets.html#stereo'},
            'aeiou.hpc': { 'aeiou.hpc.HostPrinter': 'https://drscotthawley.github.io/aeiou/hpc.html#hostprinter',
                           'aeiou.hpc.freeze': 'https://drscotthawley.github.io/aeiou/hpc.html#freeze',
                           'aeiou.hpc.get_accel_config': 'https://drscotthawley.github.io/aeiou/hpc.html#get_accel_config',
                           'aeiou.hpc.n_params': 'https://drscotthawley.github.io/aeiou/hpc.html#n_params',
                           'aeiou.hpc.save': 'https://drscotthawley.github.io/aeiou/hpc.html#save'},
            'aeiou.spectrofu': { 'aeiou.spectrofu.main': 'https://drscotthawley.github.io/aeiou/spectrofu.html#main',
                                 'aeiou.spectrofu.process_one_file': 'https://drscotthawley.github.io/aeiou/spectrofu.html#process_one_file',
                                 'aeiou.spectrofu.save_stft': 'https://drscotthawley.github.io/aeiou/spectrofu.html#save_stft'},
            'aeiou.viz': { 'aeiou.viz.audio_spectrogram_image': 'https://drscotthawley.github.io/aeiou/viz.html#audio_spectrogram_image',
                           'aeiou.viz.embeddings_table': 'https://drscotthawley.github.io/aeiou/viz.html#embeddings_table',
                           'aeiou.viz.pca_point_cloud': 'https://drscotthawley.github.io/aeiou/viz.html#pca_point_cloud',
                           'aeiou.viz.plot_jukebox_embeddings': 'https://drscotthawley.github.io/aeiou/viz.html#plot_jukebox_embeddings',
                           'aeiou.viz.print_stats': 'https://drscotthawley.github.io/aeiou/viz.html#print_stats',
                           'aeiou.viz.proj_pca': 'https://drscotthawley.github.io/aeiou/viz.html#proj_pca',
                           'aeiou.viz.spectrogram_image': 'https://drscotthawley.github.io/aeiou/viz.html#spectrogram_image',
                           'aeiou.viz.tokens_spectrogram_image': 'https://drscotthawley.github.io/aeiou/viz.html#tokens_spectrogram_image'}}}